//***************************************************************************
//  Author(s)...: Chiper
//  URL(s)......: digitalchip.ru
//  Device(s)...: MSP430 LaunchPad
//  Description.: Фотосенсор на светодиоде
//  Data........: 22.06.2013
//***************************************************************************
//  Notice: 
// Пример исползования светодиода в качестве фотодатчика
// Схема подключения:
//
//           +   PIN1  (P1.3)
//           |
//           _
//          | |  Резистор 50-100Ом
//          | |
//           - 
//           |
//           |
//         -----  LED
//          \ /   любой светодиод, можно, например 5мм
//         -----  хорошо подойдет в прозрачном пластиковом корпусе 
//           |
//           |
//           + PIN2  (P1.4)
//
// мы будем подавать положительное напряжение на PIN2 и 
// низкое напряжение в PIN1. Т.о. на светодиод будет подано обратное напряжение.
// Разумеется, светодиод светиться не будет, но будет заряжаться паразитная ёмкость
// соединения светодиода и ног микроконтроллера LaunchPad. 
// 
// Потом мы отключаем выход с PIN1 и считаем
// за какой промежуток времени заряд разрядится через светодиод.
// Причём скорость разряда зависит от освещённости светодиода.
// Чем ярче свет, тем быстрее паразитная ёмкость будет разряжаться на PIN2. 
// 
// Т.о. сразу виден недостаток идеи - в темноте время разряда может быть довольно большим.
//***************************************************************************
//
// Настройки 
#define LED_N_SIDE P1_3
#define LED_P_SIDE P1_4

#define LED_NIGHT   RED_LED  
#define LED_DAY     GREEN_LED  

#define BLINK_SPEED     15
#define LIGHT_THRESHOLD 1500

//***************************************************************************

#define STATUS_NIGHT 0
#define STATUS_DAY   1

unsigned int STATUS = 0;
unsigned int i=0;

void setup()
{
  Serial.begin(9600);
  Serial.println("LIGHT SENSOR TEST TIMING"); 
  Serial.println("http://digitalchip.ru");
  Serial.println("-------------------------");
  
  pinMode(LED_NIGHT,OUTPUT);
  pinMode(LED_DAY,OUTPUT);
}

void loop()
{
 
  unsigned int j;
 
  // Обеспечиваем обратное включение светодиода, заряжая паразитную ёмкость 
  // ног микроконтроллера и светодиода
  pinMode(LED_N_SIDE,OUTPUT);
  pinMode(LED_P_SIDE,OUTPUT);
  digitalWrite(LED_N_SIDE,HIGH);
  digitalWrite(LED_P_SIDE,LOW);

  // Изолируем pin 2 от светодиода
  pinMode(LED_N_SIDE,INPUT);
  digitalWrite(LED_N_SIDE,LOW);  // Отключаем втроенный в МК подтягивающий резистор

  // Считаем сколько требуется времени на разряд до логического нуля
  j=0;
  while(digitalRead(LED_N_SIDE)!=0) j++;
  
  // Определяем, превысило ли время указанный порог и принимаем решение, какой
  // светодиод зажечь
  if (j < LIGHT_THRESHOLD) STATUS=STATUS_DAY; else STATUS=STATUS_NIGHT;

  // Выводим результат измерения на UART
  Serial.println(j);

  // Гасим светодиоды, которые обозначают превысил уровень освещенности порог
  digitalWrite(LED_NIGHT,LOW);
  digitalWrite(LED_DAY,LOW);
  // Если превысили порог, зажигаем зеленый светодиод, иначе - красный
  if (STATUS) digitalWrite(LED_DAY,HIGH); else digitalWrite(LED_NIGHT,HIGH);

  // Мигаем светодиодом-сенсором
  // Выводы на выход
  pinMode(LED_N_SIDE,OUTPUT);
  pinMode(LED_P_SIDE,OUTPUT);
  digitalWrite(LED_N_SIDE,LOW);
  digitalWrite(LED_P_SIDE,LOW);

  // Сичитаем задержку и подаем на светодиод прямое напрядение
  i++;
  if (i > BLINK_SPEED) i=0;
  if (i == BLINK_SPEED) digitalWrite(LED_P_SIDE,HIGH);
  
  delay(100);      // Пауза
}

